"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.output = void 0;

var _fs = _interopRequireDefault(require("fs"));

var _lodash = _interopRequireDefault(require("lodash.upperfirst"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable prefer-template */
// ====================================
// Main
// ====================================
const output = async (schema, options, utils) => {
  const raw = writeTypes(utils.preprocessedSchema);
  if (options.file) _fs.default.writeFileSync(options.file, raw, 'utf8');
  return raw;
}; // ====================================
// Flow writer
// ====================================


exports.output = output;

const writeTypes = ({
  models
}) => {
  let out = '';
  Object.keys(models).forEach(modelName => {
    out += writeType(models, modelName);
  });
  return out;
};

const writeType = (models, modelName) => {
  const {
    description,
    fields = {},
    relations = {}
  } = models[modelName];
  const upperModelName = (0, _lodash.default)(modelName);
  const allSpecs = [];
  Object.keys(fields).forEach(fieldName => {
    allSpecs.push(writeField(fieldName, fields[fieldName]));
  });
  Object.keys(relations).forEach(fieldName => {
    const relation = relations[fieldName];
    if (relation.isInverse) return;
    allSpecs.push(writeField(relation.fkName, relation));
  });
  const contents = allSpecs.length ? `\n  ${allSpecs.join('\n  ')}\n` : '';
  return '' + `// ${upperModelName}\n` + (description ? `// ${description}\n` : '') + `type ${upperModelName} = {${contents}};\n\n`;
};

const writeField = (name, specs) => {
  const isRequired = specs.isRequired ? '' : '?';
  let typeStr = writeFieldType(specs.type);
  if (specs.isPlural) typeStr = `Array<${typeStr}>`;
  const comment = specs.description ? `  // ${specs.description}` : '';
  return `${name}${isRequired}: ${typeStr},${comment}`;
};

const writeFieldType = type => {
  if (type === 'uuid') return 'string';
  if (type === 'json') return 'any';
  if (type === 'date') return 'Date';
  return type;
}; // ====================================
// Public
// ====================================